// Define the COVID-19 data
const data = {
  name: 'World',
  children: [
    {
      name: 'Asia',
      value: 12,
      type: 'red',
      level: 'purple',
      children: [
        {
          name: 'Japan',
          value: 10,
          type: 'yellow',
          level: '#863a9f',
          children: [
            {
              name: 'Tourists - 200008',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'current migrants - 28907',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'immigrants - 23570',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
        {
          name: 'Korea',
          value: 7,
          type: 'yellow',
          level: '#863a9f',
          children: [
            {
              name: 'Tourists - 100000',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'current migrants - 6789',
              value: 7,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'immigrants - 2678',
              value: 7,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
      ],
    },
    {
      name: 'Australia',
      value: 10,
      type: 'pink',
      level: 'orange',
      children: [
        {
          name: 'Melbourne',
          value: 8,
          type: 'Silver',
          level: '#863a9f',
          children: [
            {
              name: 'Tourists - 500000',
              value: 8,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'current migrants - 67890',
              value: 8,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'immigrants - 6758',
              value: 8,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
        {
          name: 'Sydney',
          value: 2,
          type: 'pink',
          level: '#863a9f',
          children: [
            {
              name: 'Tourists - 80000',
              value: 4,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'current migrants - 1234',
              value: 2,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'immigrants - 678',
              value: 2,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
      ],
    },
    {
      name: 'South America',
      value: 8,
      type: 'silver',
      level: '9',
      children: [
        {
          name: 'Bolivia',
          value: 4,
          type: 'indigo',
          level: '#863a9f',
          children: [
            {
              name: 'Tourists - 900000',
              value: 7,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'current migrants - 123450',
              value: 4,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'immigrants - 6780',
              value: 3,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
        {
          name: 'Mexico',
          value: 4,
          type: 'black',
          level: '#3a9f7e',
          children: [
            {
              name: 'Tourits - 786980',
              value: 10,
              type: '#3a9f7e',
              level: '#863a9f',
            },
            {
              name: 'current migrants - 123450',
              value: 8,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'immigrants - 67809',
              value: 3,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
      ],
    },
  ],
};

// Define width, height, and margin of the tree
const margin = {
  top: 14,
  right: 300,
  bottom: 300,
  left: 100,
};
const width = 950 - margin.left - margin.right;
const height = 850 - margin.top - margin.bottom;

const tree = d3.tree().size([height, width]);

// Assign hierarchical structure to classify the data
let nodes = d3.hierarchy(data, (node) => node.children);

// Provide data connection for nodes to their respective position in the tree
nodes = tree(nodes);

// Check if SVG already exists
const svgExists = d3.select('svg').empty();

// Add an svg element if it doesn't exist
if (svgExists) {
  const svg = d3
    .select('body')
    .append('svg')
    .attr('width', width + margin.left + margin.right)
    .attr('height', height + margin.top + margin.bottom);
  const g = svg
    .append('g') // appends group
    .attr(
      'transform',
      'translate(' + margin.left + ',' + margin.top + ')',
    );

  // Define links between nodes
  const link = g
    .selectAll('.link')
    .data(nodes.descendants().slice(1))
    .enter()
    .append('path')
    .attr('class', 'link')
    .style('stroke', (d) => d.data.level)
    .attr('d', (d) => {
      return (
        'M' +
        d.y +
        ',' +
        d.x +
        'C' +
        (d.y + d.parent.y) / 2 +
        ',' +
        d.x +
        ' ' +
        (d.y + d.parent.y) / 2 +
        ',' +
        d.parent.x +
        ' ' +
        d.parent.y +
        ',' +
        d.parent.x
      );
    });

  // Declare a group for each node present in the tree
  const node = g
    .selectAll('.node')
    .data(nodes.descendants())
    .enter()
    .append('g')
    .attr(
      'class',
      (d) =>
        'node' +
        (d.children ? ' node--internal' : ' node--leaf'),
    )
    .attr(
      'transform',
      (d) => 'translate(' + d.y + ',' + d.x + ')',
    );

  // Define shape of the node
  node
    .append('circle')
    .attr('r', (d) => d.data.value)
    .style('stroke', (d) => d.data.type)
    .style('fill', (d) => d.data.level);

  // Add a new node to the list
  node
    .append('text')
    .attr('dy', '.60em')
    .attr('x', (d) =>
      d.children
        ? (d.data.value + 5) * -1
        : d.data.value + 5,
    )
    .attr('y', (d) =>
      d.children && d.depth !== 0 ? -(d.data.value + 5) : d,
    )
    .style('text-anchor', (d) =>
      d.children ? 'end' : 'start',
    )
    .text((d) => d.data.name);
}
